import { Database } from 'bun:sqlite';
import { DB_PATH } from './config';
import { logger } from './globals';
import { readdir, readFile } from "fs/promises";
import { join } from "path";

export let FTS5Enabled = true;

export const initializeDB = () => {
  logger.info("Checking fts");
  const ftstest = db.query(`pragma compile_options;`);
  const result = ftstest.all() as { compile_options: string }[];
  if (result.find((o) => o["compile_options"].includes("ENABLE_FTS5"))) {
    logger.info("FTS5 is enabled");
  } else {
    logger.info("FTS5 is not enabled. Attempting to load...");
    try {
      db.loadExtension('./fts5');
    } catch (e) {
      logger.warn("Failed to load FTS5 extension. Disabling FTS5");
      FTS5Enabled = false;
    }
  }

  return FTS5Enabled;
}

export const migrate = async () => {
  logger.info(`Checking for migrations...`);
  const result = db.query(`SELECT name FROM sqlite_master WHERE type='table' AND name='meta'`);
  if (result.all().length === 0) {
    logger.info(`Creating meta table...`);
    db.run(`CREATE TABLE meta (version INTEGER)`);
    db.run(`INSERT INTO meta (version) VALUES (0)`);
  }

  const version = db.query(`SELECT version FROM meta`).get() as { version: number };
  logger.info(`Migration version: ${version.version}`);
  // we are in bun.js. use its API's to read the file list.
  logger.info(`Searching for migrations in ${join(__dirname, "migrations")}`);
  const files = await readdir(join(__dirname, "migrations"));

  for (const file of files) {
    const [fileVersion, ...rest] = file.split("_");
    logger.info(`Found migration ${fileVersion}`);
    if (fileVersion && Number(fileVersion) > version.version) {
      logger.info(`Running migration ${file}`);
      const sql = new TextDecoder().decode(await readFile(join(__dirname, `migrations/${file}`)));
      db.run(sql);
      const query = db.query(`UPDATE meta SET version = $version`);
      const res = query.run({ $version: Number(fileVersion) })
      logger.info(`Migration ${file} done`);
    }
  }
  logger.info(`Migrations done`);
}

logger.info(`Loading database at ${DB_PATH}`);

export const db = new Database(DB_PATH);


initializeDB();
migrate();